#include <cdefBF533.h>
#include <ccblkfn.h>
#include <sys/exception.h>
#include "BlinkLED.h"

#define SSYNC				asm ("ssync;")

// ISR Prototypes
//EX_INTERRUPT_HANDLER(EVT_IVG8_Entry);
EX_INTERRUPT_HANDLER(EVT_IVG9_Entry); 

section("L1_code") void Open_Peripherals(void)
{
	//Configure System
  	Setup_Interrupt_Vectors();	// initialize used handlers
  	Configure_SDRAM();			// set up SDRAM for access
//	Configure_Flash();			// set flash up for LED access
	Configure_Flags();			// setup EZKIT switches as interrupt-generating inputs
	Configure_Interrupts();
}


/************************************************************************
 * INTERRUPT VECTORS USED												*
 ************************************************************************/
section("L1_code") void Setup_Interrupt_Vectors(void)

{
	*pSIC_IAR2 = 0x77772777;	// Assign 0x2 (IVG9)  To PFA Interrupt
	
//	register_handler (ik_ivg8, EVT_IVG8_Entry); // Default Real Time Clock Handler (Int8)
	register_handler (ik_ivg9, EVT_IVG9_Entry); // User-Assigned PFA Handler (Int9)
}

/************************************************************************
 * SETUP SDRAM SO THAT WE CAN READ AND WRITE IT							*
 ************************************************************************/    
//section("sdram0") void Configure_SDRAM(void)
section ("L1_code") void Configure_SDRAM(void)
{
    
/*
	*pEBIU_SDRRC = 0x0817;		//SDRAM Refresh Rate Control Register
	*pEBIU_SDBCTL = 0x0013;		//SDRAM Memory Bank Control Register

	*pEBIU_SDGCTL = 0x0091998D;	// SDRAM Memory Global Control Register
	ssync();
	
*/
	VDK_PushUnscheduledRegion();
	VDK_PushCriticalRegion();
	
    unsigned short i=0, j=0, k;
//	*pEBIU_AMBCTL0	= 0xFFC2FbF0;  
//	*pEBIU_AMBCTL1	= 0x7bb07bb0;   // set timing for writing to GPIO registers on CS1 

	*pEBIU_AMBCTL0	= 0xFFC2FFC2;  
	*pEBIU_AMBCTL1	= 0xFFC2FFC2;
	*pEBIU_AMGCTL	= 0x000f;
	SSYNC;

// test code to see what is loaded into VR_CTL	
//*pVR_CTL = (*pVR_CTL & 0xFFFC) | 0x8100;	
//	idle();
//	idle();
	
*( (volatile unsigned short *)(0x20100000) ) = 0xFF40;	// hold 1.2V core  and 1.8V rail on
k = *(volatile unsigned short *)(0x20000000);
SSYNC;

// added this to enable code without power control which should be really handling this   
*( (volatile unsigned short *)(0x20100000) ) = 0xFFD0;	// also turn on 2.8V rail and 5V and 9V
k = *(volatile unsigned short *)(0x20000000);  // old = 0xFFD0
SSYNC;



// Set PF9 and PF10 to set lowest power state ie drive out 11 which should give 0.80V 
// initial both tristate 1.05V 
// PPF10=0 and PF9=1 gives 1.17 which is sufficient for 400MHz on BF533L parts

    *pFIO_FLAG_C    |= (PF10 | PF9);        // value set 1.27V 

//	*pFIO_FLAG_C    |= PF10;                // value set 1.17 
//	*pFIO_FLAG_S    |= PF9;                     
    *pFIO_DIR       |= (PF10 | PF9);        // Set core control PF pins as output 
    SSYNC;   
   
 
    
    

// enable SDRAM clock control signal to pass 

*( (volatile unsigned short *)(0x20100000) ) = 0xFBD0;	
k = *(volatile unsigned short *)(0x20000000); //dummy read
SSYNC;

// Set PF11 to enable the high order GPO pins to drive out ( this include SDRAM enable control)
    *pFIO_DIR       |= PF11;        // Set PF11 as output 
    *pFIO_FLAG_S    |= PF11;        // Set high 
SSYNC;




    
		//SDRAM Refresh Rate Control Register
		*pEBIU_SDRRC = 0x00000817;	
		SSYNC;

		//SDRAM Memory Bank Control Register
		//  *pEBIU_SDBCTL = 0x00000011;  //16MByte 9 bit column addr 
		//	*pEBIU_SDBCTL = 0x00000013;  //32MByte 9 bit column addr
			*pEBIU_SDBCTL = 0x00000025;  //64MByte 10 bit column addr
		
		SSYNC;

		//SDRAM Memory Global Control Register	

		*pEBIU_SDGCTL = 0x1491998d;	// mobile sdram 0x1491998d

		SSYNC;
		
		VDK_PopCriticalRegion();	
		VDK_PopUnscheduledRegion();

}

/************************************************************************
 * SETUP FLASH SO THAT WE HAVE ACCESS TO LEDs VIA THE FLASH				*
 ************************************************************************/
section("L1_code") void Configure_Flash(void)
{
    
    
	/* Initalize EBIU control registers. */	

	*pEBIU_AMBCTL0 = 0x78B078B0;
	*pEBIU_AMBCTL1 = 0x78B078B0;
	*pEBIU_AMGCTL |= 0xF;
	ssync();



/* (JWS) Not needed in the PMP

// initialize flash A csio regs (port A and B)
    // clear data registers
	*pFlashA_PortB_Data = 0x0;

    // direction control registers
	*pFlashA_PortB_Dir = (unsigned char)0xFFFF;      // set dir=output
	ssync();
*/
}

/************************************************************************
 * SETUP FLAGS TO ACCEPT PUSH-BUTTON INPUTS AS INTERRUPTS				*
 ************************************************************************/
section("L1_code") void Configure_Flags()
{
// (JWS) Old settings
/*
   *pFIO_INEN = 0x0400;			// PF10 input enable
   *pFIO_EDGE = 0x0400;			// PF10 edge sensitive
   *pFIO_MASKA_D = 0x0400;		// PF10 Interrupt A
   *pFIO_DIR = 0xFBFF;			// PF15-11, 9-0 Outputs  
*/
// (JWS) Added new switch for PMP
   *pFIO_INEN = 0x0100;			// PF8 input enable
   *pFIO_EDGE = 0x0100;			// PF8 edge sensitive (hardware shows it as a neg active edge)
   *pFIO_MASKA_D = 0x0100;		// PF8 Interrupt A
   
   *pFIO_DIR = 0xFEFF;			// PF15-9, 7-0 Outputs 
   
// (JWS) No profiling for now  
//   *pFIO_FLAG_S = 0x0080;		// set PF7 high for profiling purposes
}

/************************************************************************
 * CONFIGURE REAL-TIME-CLOCK TO 1 HZ TICKS AND ENABLE STOPWATCH			*
 ************************************************************************/
section("L1_code") void Configure_RTC()
{

	*pRTC_PREN = 0x0;
	
	while (!(*pRTC_ISTAT & 0x8000));
	
	*pRTC_SWCNT = 0x7fff;			// set StopWatch to (80/32768) seconds
									// (JWS) old = 0x0050
	*pRTC_ISTAT = CLEAR_RTC_IRQS;	// clear any pending IRQs
	*pRTC_ICTL = 0x1;				// set Stopwatch interrupt enable
	
	while (!(*pRTC_ISTAT & 0x8000));

}

/************************************************************************
 * ENABLE WAKE-UP INTERRUPTS, CORE INTERRUPTS, AND SYSTEM INTERRUPTS	*
 ************************************************************************/
section("L1_code") void Configure_Interrupts(void)
{
   *pSIC_IWR = 0x01;				// Enable PLL interrupt for wakeup.
 
   *pSIC_IMASK = 0x00080000;		// Unmask PFA (Bit 19) Interrupt
   									// Enable GPF Interrupt A.
}

